/*
 *  Copyright  2004  Day Choi <eo1120@yahoo.co.kr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _CHATWINDOW_H
#define _CHATWINDOW_H

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef __cplusplus
}
#endif

#include <gtkmm/window.h>
#include <queue>
#include <list>
#include "SockChannel.h"
#include "BuddyModel.h"
#include "InviteWindow.h"
#include "smartpointer.h"
#include "Utility.h"

typedef smart_pointer<SockChannel> SockChannelPtr;
typedef smart_pointer<InviteWindow> InviteWindowPtr;

enum { RING, CALL };
	
class ChatWindow : public Gtk::Window
{  
public:
	ChatWindow();
	ChatWindow(const std::string& _buddyid);
	~ChatWindow();
	void connect(const std::string ip, const int port, const std::string cki, const int type);
	void request_display_picture(const std::string& object);

private:
	std::string buddyid;
	std::list<std::string> onsessionbuddylist;
	bool chatconnection;
	bool is_offline;
	
	SockChannelPtr sockchannel;
	InviteWindowPtr invitewindow;
	bool is_invitewindow;
	
	Glib::RefPtr<Gtk::TextBuffer> entrybuffer;
	Gtk::TextView *entry;
	Glib::RefPtr<Gtk::TextBuffer> textbuffer;
	Gtk::TextView *text;
	Glib::RefPtr<Gtk::TextBuffer::Mark> textmark;
	
	std::queue<std::string> messagequeue;
	
	unsigned long int m_msgIdentifier;
	unsigned long int m_sessionId;
	unsigned long int m_totalDataSize;
	unsigned long int m_offset;
	void send_p2p_message( const std::string& dataMessage );
	
	bool io_handler(Glib::IOCondition cond);
	bool parse_stream(const std::string& data);
	bool on_entry_button_press_event(GdkEventButton* event);
	bool on_entry_key_press_event(GdkEventKey* event);
	void on_invite_clicked();
	
};

#endif /* _CHATWINDOW_H */
