 /*
 *  Copyright  2004  Day Choi <eo1120@yahoo.co.kr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "InviteWindow.h"
#include "Utility.h"
#include "global.h"

InviteWindow::InviteWindow(const smart_pointer<SockChannel>& _sockchannel,
				std::list<std::string>& _onsessionbuddylist) :
	sockchannel(_sockchannel),
	onsessionbuddylist(_onsessionbuddylist)
{
	Gtk::TreeViewColumn *const column = new Gtk::TreeViewColumn();	
	CellRendererIcon *const iconrenderer = new CellRendererIcon();
	column->pack_start(*iconrenderer, false);
	column->add_attribute(iconrenderer->property_pixbuf(), mainwindow->buddycolumnmodel.icon);
	column->add_attribute(iconrenderer->property_visible(), mainwindow->buddycolumnmodel.icon_visible);
	CellRendererNick *const nickrenderer = new CellRendererNick();
	column->pack_start(*nickrenderer, true);
	column->add_attribute(nickrenderer->property_markup(), mainwindow->buddycolumnmodel.nick);
	
	treeview = Gtk::manage(new class Gtk::TreeView());
	treeview->set_model(mainwindow->treemodel);
	treeview->append_column(*Gtk::manage(column));
	treeview->set_name("treeview");
	treeview->set_flags(Gtk::CAN_FOCUS);
	treeview->set_headers_visible(false);
	treeview->set_rules_hint(false);
	treeview->set_reorderable(false);
	treeview->set_enable_search(false);
	tree = treeview->get_selection();
	tree->set_mode(Gtk::SELECTION_MULTIPLE);
	tree->unselect_all();
	treeview->show();
	
	Gtk::ScrolledWindow *scrolledwindow = Gtk::manage(new class Gtk::ScrolledWindow());
	scrolledwindow->set_name("scrolledwindow2");
	scrolledwindow->set_flags(Gtk::CAN_FOCUS);
	scrolledwindow->set_shadow_type(Gtk::SHADOW_NONE);
	scrolledwindow->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	scrolledwindow->property_window_placement().set_value(Gtk::CORNER_TOP_LEFT);
	scrolledwindow->add(*treeview);
	scrolledwindow->show();
	
	Gtk::HSeparator *hseparator = Gtk::manage(new class Gtk::HSeparator());
	hseparator->show();
	
	Gtk::Button *button1 = Gtk::manage(new class Gtk::Button(Gtk::StockID("gtk-cancel")));
	button1->set_flags(Gtk::CAN_FOCUS);
	button1->set_relief(Gtk::RELIEF_NORMAL);
	button1->show();

	Gtk::Button *button2 = Gtk::manage(new class Gtk::Button(Gtk::StockID("gtk-ok")));
	button2->set_flags(Gtk::CAN_FOCUS);
	button2->set_relief(Gtk::RELIEF_NORMAL);
	button2->show();

	Gtk::HButtonBox *hbuttonbox = Gtk::manage(new class Gtk::HButtonBox(Gtk::BUTTONBOX_END, 0));
	hbuttonbox->pack_start(*button1);
	hbuttonbox->pack_start(*button2);
	hbuttonbox->show();

	Gtk::VBox *vbox = Gtk::manage(new class Gtk::VBox(false, 0));
	vbox->pack_start(*scrolledwindow);
	vbox->pack_start(*hseparator, Gtk::PACK_SHRINK, 0);
	vbox->pack_start(*hbuttonbox, Gtk::PACK_SHRINK, 0);
	vbox->show();

	this->set_name("InviteWindow");
	this->set_title(_("invite"));
	this->set_modal(false);
	this->property_window_position().set_value(Gtk::WIN_POS_NONE);
	this->set_resizable(true);
	this->set_default_size(350, 200);
	this->property_destroy_with_parent().set_value(false);
	this->add(*vbox);
	/*
	std::map<const unsigned int, GroupInfo>::iterator grouppos;
	for(grouppos = group.begin(); grouppos != group.end(); ++grouppos)
	{
		Gtk::TreeModel::Row row = *(treemodel->append());		
		std::string groupname = grouppos->second.groupname;
		
		std::string mark;
		mark = "<span weight='bold'>";
		mark += url_decode(groupname);
		mark += "</span>";
			
		row[buddycolumnmodel.groupnum] = grouppos->first;
		row[buddycolumnmodel.icon_visible] = false;
		row[buddycolumnmodel.nick] = mark;
	}
	
	std::map<const std::string, BuddyInfo>::iterator buddypos;
	for(buddypos=buddy.begin(); buddypos != buddy.end(); ++buddypos)
	{
		if (buddypos->second.status != FLN)
		{
			std::list<std::string>::iterator pos;
			pos = find(onsessionbuddylist.begin(), onsessionbuddylist.end(), buddypos->first);
			if (pos == onsessionbuddylist.end()) {
				Gtk::TreeModel::Row row = *(treemodel->append());
				row[buddycolumnmodel.id] = buddypos->first;			
				row[buddycolumnmodel.groupnum] = buddypos->second.groupnum;
				row[buddycolumnmodel.icon_visible] = true;
				row[buddycolumnmodel.nick] = url_decode(buddypos->second.nick);
				row[buddycolumnmodel.icon] = buddyimage[buddypos->second.status];
			}
		}
	}
	*/
	
	treeview->expand_all();	
	treeview->signal_row_activated().connect(SigC::slot(*this, &InviteWindow::on_row_activated));
	button1->signal_clicked().connect(SigC::slot(*this, &InviteWindow::on_cancel_clicked));
	button2->signal_clicked().connect(SigC::slot(*this, &InviteWindow::on_ok_clicked));
	this->signal_delete_event().connect(SigC::slot(*this, &InviteWindow::on_window_delete_event));
}

InviteWindow::~InviteWindow()
{}
	
void
InviteWindow::on_ok_clicked()
{
	Glib::ListHandle<Gtk::TreeModel::Path, Gtk::TreePath_Traits> selected_list = tree->get_selected_rows();
	Glib::Container_Helpers::ListHandleIterator<Gtk::TreePath_Traits> pos(selected_list.begin());
	for(; pos != selected_list.end(); ++pos)
	{
		const Gtk::TreeModel::iterator iter = mainwindow->treemodel->get_iter(*pos);
		std::string id;
		Gtk::TreeModel::Row row = *iter;
		id = row[mainwindow->buddycolumnmodel.id];
		
		if (row[mainwindow->buddycolumnmodel.icon_visible] == false)  //is group
		{
		}
		else if (buddy[id].status != FLN) 
		{
			std::list<std::string>::iterator buddyinfo_pos;
			buddyinfo_pos = find(onsessionbuddylist.begin(), onsessionbuddylist.end(), id);
			if (buddyinfo_pos == onsessionbuddylist.end()) {
				sockchannel->sendCommand("CAL  %s\r\n", id.c_str());
				std::cout << "invite : " << id << std::endl;	
			tree->unselect_all();
			this->hide();
			}
		} 
	}
	
}

bool 
InviteWindow::on_window_delete_event(GdkEventAny *event)
{
	tree->unselect_all();
	this->hide();
	return true;
}

void
InviteWindow::on_cancel_clicked()
{
	tree->unselect_all();
	this->hide();
}

void
InviteWindow::on_row_activated(const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn* column) 
{
	const Gtk::TreeModel::iterator iter = mainwindow->treemodel->get_iter(path);
	if (iter)
	{
		std::string id;
		Gtk::TreeModel::Row row = *iter;
		id = row[mainwindow->buddycolumnmodel.id];
		
		if (row[mainwindow->buddycolumnmodel.icon_visible] == false)  //is group
		{
			if ( treeview->row_expanded(path) ) {
				treeview->collapse_row(path);
			} else {
				treeview->expand_row(path, true);
			}
		}
		else if (buddy[id].status != FLN) 
		{
			std::list<std::string>::iterator pos;
			pos = find(onsessionbuddylist.begin(), onsessionbuddylist.end(), id);
			if (pos == onsessionbuddylist.end()) {
				sockchannel->sendCommand("CAL  %s\r\n", id.c_str());
				std::cout << "invite : " << id << std::endl;	
			tree->unselect_all();
			this->hide();
			}
		} 
	}
}
