/*
 *  Copyright  2004  Day Choi <eo1120@yahoo.co.kr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "LoginWindow.h"
#include "MainWindow.h"
#include "SockChannel.h"

#include <glibmm/main.h>
#include <iostream>
#include "global.h"

LoginWindow::LoginWindow()
{
	Gtk::Label* label1 = Gtk::manage(new class Gtk::Label(_("E-mail Address:")));
	combo1 = Gtk::manage(new class Gtk::Combo());
	Gtk::Label* label2 = Gtk::manage(new class Gtk::Label(_("Password:")));
	entry1 = Gtk::manage(new class Gtk::Entry());
	Gtk::Button* button1 = Gtk::manage(new class Gtk::Button(Gtk::StockID("gtk-preferences")));
	Gtk::Button* button2 = Gtk::manage(new class Gtk::Button(Gtk::StockID("gtk-ok")));
	Gtk::HButtonBox* hbuttonbox1 = Gtk::manage(new class Gtk::HButtonBox(Gtk::BUTTONBOX_SPREAD, 0));
	Gtk::VBox *vbox2 = Gtk::manage(new class Gtk::VBox(false, 0));
	label1->set_name("label1");
	label1->set_alignment(0,0);
	label1->set_padding(0,0);
	label1->set_justify(Gtk::JUSTIFY_LEFT);
	label1->set_line_wrap(false);
	label1->set_use_markup(false);
	label1->set_selectable(false);
	combo1->get_entry()->set_name("combo-entry1");
	combo1->get_entry()->set_flags(Gtk::CAN_FOCUS);
	combo1->get_entry()->set_visibility(true);
	combo1->get_entry()->set_editable(true);
	combo1->get_entry()->set_max_length(0);
	combo1->get_entry()->set_has_frame(true);
	combo1->get_entry()->set_activates_default(false);
	combo1->set_name("combo1");
	combo1->set_case_sensitive(false);
	label2->set_name("label2");
	label2->set_alignment(0,0);
	label2->set_padding(0,0);
	label2->set_justify(Gtk::JUSTIFY_LEFT);
	label2->set_line_wrap(false);
	label2->set_use_markup(false);
	label2->set_selectable(false);
	entry1->set_name("entry1");
	entry1->set_flags(Gtk::CAN_FOCUS);
	entry1->set_visibility(true);
	entry1->set_editable(true);
	entry1->set_max_length(0);
	entry1->set_has_frame(true);
	entry1->set_activates_default(false);
	entry1->set_visibility(false);
	button1->set_name("button1");
	button1->set_flags(Gtk::CAN_FOCUS);
	button1->set_relief(Gtk::RELIEF_NONE);
	button2->set_name("button2");
	button2->set_flags(Gtk::CAN_FOCUS);
	button2->set_relief(Gtk::RELIEF_NONE);
	hbuttonbox1->set_name("hbuttonbox1");
	hbuttonbox1->pack_start(*button1);
	hbuttonbox1->pack_start(*button2);
	vbox2->set_name("vbox2");
	vbox2->set_border_width(9);
	vbox2->pack_start(*label1, Gtk::PACK_SHRINK, 0);
	vbox2->pack_start(*combo1, Gtk::PACK_SHRINK, 0);
	vbox2->pack_start(*label2, Gtk::PACK_SHRINK, 0);
	vbox2->pack_start(*entry1, Gtk::PACK_SHRINK, 0);
	vbox2->pack_start(*hbuttonbox1);
	this->set_name("LoginWindow");
	this->set_title(_("Login"));
	this->set_modal(false);
	this->property_window_position().set_value(Gtk::WIN_POS_NONE);
	this->set_resizable(false);
	this->property_destroy_with_parent().set_value(false);
	this->add(*vbox2);
	label1->show();
	combo1->show();
	label2->show();
	entry1->show();
	button1->show();
	button2->show();
	hbuttonbox1->show();
	vbox2->show();
	
	entry1->signal_activate().connect( SigC::slot(*this, &LoginWindow::on_activate_entry) );
	button2->signal_clicked().connect( SigC::slot(*this, &LoginWindow::on_activate_entry) );
	
}

LoginWindow::~LoginWindow()
{}

void
LoginWindow::connect()
{
	try
	{
		sockchannel = new SockChannel(MSN_DISPATCH_SERVER_ADDRESS, MSN_DISPATCH_SERVER_PORT);
	}
	catch (SocketException e)
	{
		std::cout << e.description() << std::endl;
		exit(0);
	}
	
	Glib::signal_io().connect(SigC::slot(*this, &LoginWindow::io_handler), sockchannel->getChannel(), Glib::IO_IN | Glib::IO_HUP);
	sockchannel->sendCommand("VER  MSNP9 MSNP8 CVR0\r\n");
}

void 
LoginWindow::on_activate_entry()
{
	// TODO:
	//
	//	check the inputed data valid ( e-mail rule )

	userinfo.id = combo1->get_entry()->get_text();
	userinfo.password = entry1->get_text();
	connect();
}

bool 
LoginWindow::io_handler(Glib::IOCondition cond)
{
	std::string data;
	try 
	{
		*sockchannel >> data;
	}
	catch (SocketException e)
	{
		std::cout << e.description() << std::endl;
		exit(0);
	}
	
	std::cout 	<< "\n---------------- loginwindow received ----------------\n" 
				<< data 
				<< "\n------------- end of loginwindow received ------------\n" <<std::endl;
	if (data.substr(0,3) == "VER")
		sockchannel->sendCommand("CVR  0x0409 winnt 5.1 i386 MSNMSGR 6.0.0602 MSMSGS %s\r\n",userinfo.id.c_str());
	
	if (data.substr(0,3) == "CVR")
	{
		sockchannel->sendCommand("USR  TWN I %s\r\n",userinfo.id.c_str());
	}
	if (data.substr(0,3) == "XFR")
	{
		char ip[16];
		int port;
		sscanf(data.c_str(), "%*s %*d %*s %[^:]:%d %*d %*s", ip , &port);
		
		mainwindow->connect(ip, port);
		mainwindow->show();
		// emit delete signal
		g_signal_emit_by_name(this->gobj(), "delete_event");
		
		return false;
	}
	
	return true;
}
