/*
 *  Copyright  2004  Day Choi <eo1120@yahoo.co.kr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "SSLSockChannel.h"
#include <iostream>

SSLSockChannel::SSLSockChannel ( std::string host, int port ) :
	SockChannel (host, port)
{
	/* Allow connections to servers that have OpenPGP keys as well.*/
	const int cert_type_priority[2] = { GNUTLS_CRT_X509, 0 };
	
	gnutls_global_init();
	
	/* X509 stuff */
	gnutls_certificate_allocate_credentials(&xcred);
	
	/* sets the trusted cas file*/
	gnutls_certificate_set_x509_trust_file(xcred, CAFILE, GNUTLS_X509_FMT_PEM);
	
	/* Initialize TLS session */
	gnutls_init(&session, GNUTLS_CLIENT);
	
	/* Use default priorities */
	gnutls_set_default_priority(session);
	gnutls_certificate_type_set_priority(session, cert_type_priority);
	
	/* put the x509 credentials to the current session*/
	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
	/* connect to the peer*/
	gnutls_transport_set_ptr( session, (gnutls_transport_ptr)m_sock);
	 
	/* Perform the TLS handshake*/
	int ret = gnutls_handshake( session);
	
	if (ret < 0) {
		std::cerr << "*** Handshake failed" << std::endl;
		gnutls_perror(ret);
	} else {
		std::cout << "*** Handshake was completed\n" << std::endl;
	}
}

SSLSockChannel::~SSLSockChannel ()
{
	gnutls_bye( session, GNUTLS_SHUT_RDWR);
	gnutls_deinit(session);
	gnutls_certificate_free_credentials(xcred);
	gnutls_global_deinit();
	std::cout << "*** ssl session end\n" << std::endl;
}

const SSLSockChannel& 
SSLSockChannel::operator << ( const std::string& str ) const
{
	if ( gnutls_record_send( session, str.c_str(), str.size()) <= 0)
	{
		throw SocketException ( "Could not write to sslsocket." );
	}

	return *this;
}

const SSLSockChannel& 
SSLSockChannel::operator >> ( std::string& str ) const
{
	unsigned int pos = 0;
	char temp_buf[SSL_MAX_BUF + 1];
	memset(temp_buf,0,SSL_MAX_BUF);
	str="";
	while((gnutls_record_recv( session, temp_buf, sizeof(temp_buf)))>0)
	{
		str.append(temp_buf);
		pos = str.find("\r\n\r\n");
		if (pos>= 0 && pos < str.size()) break;
	}
	
	str = str.substr(0, pos+4);	

	return *this;
}
