/*
 *  Copyright  2004  Day Choi <eo1120@yahoo.co.kr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "SockChannel.h"
#include "global.h"
#include <cstdlib> 

SockChannel::SockChannel ( std::string host, int port ) :
	trid(0)
{
	if ( ! Socket::create() )
	{
		throw SocketException ( "Could not create client socket." );
	}

	if ( ! Socket::connect ( host, port ) )
	{
		throw SocketException ( "Could not bind to port." );
	}
}

SockChannel::~SockChannel ()
{}

const SockChannel& 
SockChannel::operator << ( const std::string& s ) const
{
	if ( ! Socket::send ( s ) )
	{
		throw SocketException ( "Could not write to socket." );
	}

	return *this;
}

const SockChannel& 
SockChannel::operator >> ( std::string& s ) const
{
	if ( ! Socket::recv ( s ) )
	{
		throw SocketException ( "Could not read from socket." );
	}

	return *this;
}

const Glib::RefPtr<Glib::IOChannel> 
SockChannel::getChannel()
{
	channel = Glib::IOChannel::create_from_fd(m_sock);
    channel->set_flags(Glib::IO_FLAG_NONBLOCK|channel->get_flags());
	return channel;
}

int
SockChannel::sendCommand(const char *fmt, ...)
{
	int ret;
	va_list ap;
	char buf[MSN_MAX_BUF_LEN];
	std::string data;
	
	va_start(ap, fmt);
	ret = vsprintf(buf, fmt, ap);
	va_end(ap);
	
	data = buf;
	data.insert(4, itoa(++trid,10));
	std::cout << data << std::endl;
	try 
	{
		*this << data;
	}
	catch (SocketException e)
	{
		std::cerr << e.description();
	}
	return ret;
}
