/*
 *  Copyright  2004  Day Choi <eo1120@yahoo.co.kr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <glibmm/markup.h>
#include <glibmm/ustring.h>
#include <config.h>

static const unsigned char alphabet[65] =
        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

const std::string 
url_decode(const std::string& get_str)
{
	unsigned int i = 0;
	unsigned int find_pos;
	unsigned int length;
	char hex[3];
	
	std::string str = get_str;
	length = str.length();
	
	while((find_pos = str.find("%",i)) >= 0 && find_pos < length )
	{
		i = find_pos;
		strncpy(hex, str.substr(find_pos + 1, 2).c_str(), 2);
		str.erase(find_pos + 1, 2);
		str[find_pos] = strtol(hex, NULL, 16);
	}

	return Glib::Markup::escape_text(str);
}
const std::string 
url_decode_unescape(const std::string& get_str)
{
	unsigned int i = 0;
	unsigned int find_pos;
	unsigned int length;
	char hex[3];
	
	std::string str = get_str;
	length = str.length();
	
	while((find_pos = str.find("%",i)) >= 0 && find_pos < length )
	{
		i = find_pos;
		strncpy(hex, str.substr(find_pos + 1, 2).c_str(), 2);
		str.erase(find_pos + 1, 2);
		str[find_pos] = strtol(hex, NULL, 16);
	}

	return str;
}

const std::string 
url_encode(const std::string& get_str)
{
	char hex[4];
	
	std::string str = get_str;
	
	for (unsigned int i=0; i < str.length(); ++i)
	{
		if (!isalnum(str[i]))
		{
			sprintf(hex, "%%%02x", (unsigned char)str[i]);
			str.replace(i,1,hex);
		}
	}
	
	return str;
}

size_t b64Encode(const unsigned char *src, size_t srcsize,
						   char *dst, size_t dstsize)
{
	if (!dstsize) return 0;

	char* pdst = dst;
	unsigned bits;

	while(srcsize >= 3 && dstsize > 4)
	{
			bits = (((unsigned)src[0])<<16) | (((unsigned)src[1])<<8)
					| ((unsigned)src[2]);
			src += 3;
			srcsize -= 3;
			*(pdst++) = alphabet[bits >> 18];
			*(pdst++) = alphabet[(bits >> 12) & 0x3f];
			*(pdst++) = alphabet[(bits >> 6) & 0x3f];
			*(pdst++) = alphabet[bits & 0x3f];
			dstsize -= 4;
	}
	if (srcsize && dstsize > 4)
	{
			bits = ((unsigned)src[0])<<16;
			*(pdst++) = alphabet[bits >> 18];
			if (srcsize == 1)
			{
					*(pdst++) = alphabet[(bits >> 12) & 0x3f];
					*(pdst++) = '=';
			}
			else
			{
					bits |= ((unsigned)src[1])<<8;
					*(pdst++) = alphabet[(bits >> 12) & 0x3f];
					*(pdst++) = alphabet[(bits >> 6) & 0x3f];
			}
			*(pdst++) = '=';
	}
	*pdst = 0;
	return pdst-dst;
}


std::string b64Encode(const std::string& str)
{
	size_t limit = (str.length()+2)/3*4+1;  // size + null must be included
	char* buffer = new char[limit];
	
	unsigned size = b64Encode((const unsigned char *)str.c_str(), str.length(), buffer, limit);
	buffer[size] = '\0';
	
	std::string final = buffer;
	delete buffer;
	return final;
}

size_t b64Decode(const char *src,
				unsigned char *dst, size_t dstsize)
{
	char decoder[256];
	int i, bits, c;

	unsigned char *pdst = dst;

	for (i = 0; i < 256; ++i)
			decoder[i] = 64;
	for (i = 0; i < 64 ; ++i)
			decoder[alphabet[i]] = i;

	bits = 1;

	while(*src)
	{
			c = (unsigned char)(*(src++));
			if (c == '=')
			{
					if (bits & 0x40000)
					{
							if (dstsize < 2) break;
							*(pdst++) = (bits >> 10);
							*(pdst++) = (bits >> 2) & 0xff;
							break;
					}
					if (bits & 0x1000 && dstsize)
							*(pdst++) = (bits >> 4);
					break;
			}
			// skip invalid chars
			if (decoder[c] == 64)
					continue;
			bits = (bits << 6) + decoder[c];
			if (bits & 0x1000000)
			{
					if (dstsize < 3) break;
					*(pdst++) = (bits >> 16);
					*(pdst++) = (bits >> 8) & 0xff;
					*(pdst++) = (bits & 0xff);
					bits = 1;
					dstsize -= 3;
			}
	}
	return pdst-dst;
}

std::string b64Decode(const std::string& str)
{
	size_t limit = str.length()/4*3;
	unsigned char* buffer = new unsigned char[limit+1];
	
	unsigned size = b64Decode(str.c_str(), buffer, limit);
	buffer[size] = '\0';
	
	std::string final = (char*)buffer;
	delete buffer;
	return final;
}

static char *_itoa(unsigned i, char *a, unsigned r)
{
        if (i/r > 0) a = _itoa(i/r,a,r);
        *a = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"[i%r];
        return a+1;
}
char *itoa(int i, char *a, int r)
{
        r = ((r < 2) || (r > 36)) ? 10 : r;
        if (i < 0)
        {
                *a = '-';
                *_itoa(-i,a+1,r) = 0;
        }
        else *_itoa(i,a,r) = 0;
        return a;
}
std::string itoa(unsigned int i, int radix)
{
	std::string str;
	char* buffer = new char[10];
	str = itoa( i , buffer, radix);
	delete buffer;
	return str;
}
const char* itoa( char *buf, size_t cchBuf, unsigned int i)
{
		char *psz = buf + cchBuf - 1;   // Set psz to last char
		*psz = 0;                           // Set terminating null
		do
		{
				unsigned lsd = i % 10;       // Get least significant
				i /= 10;                        // Prepare for next most
				--psz;                          // Move back
				*psz = lsd + 48;
		} while(i != 0);
		return psz;
}
std::string itoa(unsigned int i)
{
	std::string str;
	char* buffer = new char[10];
	str = itoa(buffer, sizeof(char[10]), i);
	delete buffer;
	return str;
}
std::string toLower (const std::string & s)
{
	std::string d (s);
	
	std::transform (d.begin (), d.end (), d.begin (), (int(*)(int)) std::tolower);
	return d;
}  // end of tolower

// returns an upper case version of the string 
std::string toUpper (const std::string & s)
{
	std::string d (s);
	
	std::transform (d.begin (), d.end (), d.begin (), (int(*)(int)) std::toupper);
	
	return d;
}   // end of toupper


const std::string 
pixmap_dir(const std::string& get_str)
{
	return PACKAGE_PIXMAPS_DIR + get_str;
}
