// from http://gc.myscan.org/gc11/node2.html

#ifndef _SMARTPOINTER_H
#define _SMARTPOINTER_H

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef __cplusplus
}
#endif

template <class T>
class smart_pointer {
private:
     unsigned int* _count_p;
     T* _refered_object;
public:
     inline smart_pointer(void);
     inline smart_pointer(const T* x);
     inline smart_pointer(const smart_pointer& x);
     inline ~smart_pointer(void);

     inline smart_pointer& operator=(int n);
     smart_pointer& operator=(T* x);
     smart_pointer& operator=(smart_pointer& x);
     inline T& operator*() const;
     inline T* operator->() const;
     inline bool operator==(const smart_pointer& x) const;
     inline bool operator<(const smart_pointer& x) const;
     inline bool operator>(const smart_pointer& x) const;

     inline int ref_count() const;
};

template <class T>
smart_pointer<T>::smart_pointer(void)
     :_count_p(0), _refered_object(0) {}

template <class T>
smart_pointer<T>::smart_pointer(const T* x)
     :_count_p(new unsigned int(1)), _refered_object((T*) x) {}

template <class T>
smart_pointer<T>::smart_pointer(const smart_pointer<T>& x)
     :_count_p(x._count_p), _refered_object(x._refered_object)
{
     if (_refered_object)
	  ++*_count_p;
}

template <class T>
smart_pointer<T>::~smart_pointer(void)
{
     if (_refered_object && !--*_count_p) {
	  delete _refered_object;
	  delete _count_p;
     }
}

template <class T>
smart_pointer<T>& smart_pointer<T>::operator=(int n)
{
     if (_refered_object && !--*_count_p) {
	  delete _refered_object;
	  delete _count_p;
     }
     _refered_object = 0;
     _count_p = 0;
}

template <class T>
smart_pointer<T>& smart_pointer<T>::operator=(T* x)
{
     if (_refered_object) {
	  if (!--*_count_p) {
	       delete _refered_object;
	  } else {
	       _count_p = new unsigned int;
	  }
     }
     if (!_count_p) {
	  _count_p = new unsigned int;
     }
     _refered_object = x;
     *_count_p = 1;
     return *this;
}

template <class T>
smart_pointer<T>& smart_pointer<T>::operator=(smart_pointer<T>& x)
{
     if (this == &x)
	  return *this;

     if (_refered_object && !--*_count_p) {
	  delete _refered_object;
	  delete _count_p;
     }
     _refered_object = x._refered_object;
     ++*(_count_p = x._count_p);
     return *this;
}

template <class T>
T& smart_pointer<T>::operator*() const
{
     return *_refered_object;
}

template <class T>
T* smart_pointer<T>::operator->() const
{
     return _refered_object;
}

template <class T>
bool smart_pointer<T>::operator==(const smart_pointer<T>& x) const
{
     return *_refered_object == *x._refered_object;
}

template <class T>
bool smart_pointer<T>::operator<(const smart_pointer<T>& x) const
{
     return *_refered_object < *x._refered_object;
}

template <class T>
bool smart_pointer<T>::operator>(const smart_pointer<T>& x) const
{
     return *_refered_object > *x._refered_object;
}

template <class T>
int smart_pointer<T>::ref_count() const
{
     return *_count_p;
}

#endif /* _SMARTPOINTER_H */
